﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Algorithms.DataAnalysis.SymRegGrammarEnumeration {

  public abstract class Symbol {
    private readonly string stringRepresentation;

    protected Symbol(string representation) {
      stringRepresentation = representation;
    }

    public override string ToString() {
      return stringRepresentation;
    }
  }

  public class TerminalSymbol : Symbol {
    public TerminalSymbol(string representation) : base(representation) { }
  }

  public class NonterminalSymbol : Symbol {
    public List<Production> Alternatives;

    public NonterminalSymbol(string representation) : base(representation) {
      Alternatives = new List<Production>();
    }

    public void AddProduction(params Symbol[] production) {
      Alternatives.Add(new Production(production));
    }
  }

  public class VariableSymbol : NonterminalSymbol { // Convenience class
    public VariableSymbol(string representation, IEnumerable<string> variableNames) : base(representation) {
      foreach (string variableName in variableNames) {
        AddProduction(new TerminalSymbol("[" + variableName + "]"));
      }
    }
  }

  public class Production : List<Symbol> {

    public Production(params Symbol[] symbols) : base(symbols) { }

    public Production(IEnumerable<Symbol> symbols) : base(symbols) { }

    public override string ToString() {
      return string.Join(" ", this);
    }
  }
}
